// mozileUndoRedo.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Undo Redo Javascript
Mozile version 0.60

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-03-24

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	This code handles the mozileUndo and mozileRedo functions, as well as the
mozileDo function.

*********/




/********** Undo Functionality *************/

// This is the lazy way, I copy the whole BODY...
// This code still needs to be tweaked.


// Mozile Do
// Records the state of the current document into the undoStack for that document,
// and increments the undoCounter (each document gets its own stack and counter).
// All steps higher up in the stack are eliminated when a lower on e is added.
// If the undoMax number of steps is hit, then the stack is spliced.
// The undoStack includes the name of the command and the whole BODY node.

// Yes, it is a brute force solution to copy the whole BODY,
// but it is the best solution for multiple contentEditable areas,
// (doing the whole HTML messes up the scrollbar)
// and the performance seems fine.
// Of course this won't work for XML in general...

function mozileDo(name) {
	try{
		if(!mozileTarget.undoStack) {
			mozileTarget.undoStack=new Array();
			mozileTarget.undoCounter=0;
		}
		var undoStep = new Array;
		var getNode=mozileTarget.document.documentElement.childNodes[2];
		undoStep['name']=name;
		undoStep['node']=getNode.cloneNode(true);
		//undoStep['position']=getPosition(getNode);
		mozileTarget.undoStack=mozileTarget.undoStack.slice(0,mozileTarget.undoCounter);			// make sure there are no extra steps
		if(mozileTarget.undoStack.length > undoMax) {
			mozileTarget.undoStack=mozileTarget.undoStack.slice(1);			// make sure there are no extra steps
		}
		mozileTarget.undoStack.push(undoStep);
		mozileTarget.undoCounter=mozileTarget.undoStack.length;
		//alert("Undo:"+mozileTarget.undoStack.length +":"+mozileTarget.undoCounter);
		mozileUpdateUndoButtons();
	} catch(e) { alert("mozileDo(): "+e); }
} // end mozileDo



// Update Undo Buttons
// This works but is not currently used.
// It gives the user some feedback about when more undo/redo steps are available.

function updateUndoButtons() {
	var undo=document.getElementById('mozile-undo');
	var redo=document.getElementById('mozile-redo');
	if(mozileTarget.undoCounter<1) undo.setAttribute('disabled','true');
	else undo.removeAttribute('disabled');
	if(mozileTarget.undoCounter>mozileTarget.undoStack.length-2) redo.setAttribute('disabled','true');
	else redo.removeAttribute('disabled');
} // end updateUndoButtons()

// Dump Undo Stack
// Mostly for debugging

function dumpUndoStack() {
	var out="Current: "+mozileTarget.undoCounter+"\n";
	for(var i=0;i<mozileTarget.undoStack.length;i++) {
		cur=mozileTarget.undoStack[i];
		out+="["+i+"] "+cur['name'] +": "+ cur['node'].tagName +"\n";
	}
	alert(out);
} // end dumpUndoStack()




// Mozile Undo
// Moves doen the undoStack from the curent undoCounter,
// replacing the current BODY node with the one from the stack.
// Behaves a bit strangely around the undoMax limit.

function mozileUndo() {
	if(!mozileTarget.undoStack) {
		mozileTarget.undoStack=new Array();
		mozileTarget.undoCounter=0;
		return;
	}
	var undoStack=mozileTarget.undoStack;				// it is safe to copy this, since it will not be manipulated

	if(mozileTarget.undoCounter<1) {
		//alert("Nothing to Undo!");
		return;
	}
	if(mozileTarget.undoCounter==undoStack.length) {
		//alert("Current");
		mozileDo('Current');
 		mozileTarget.undoCounter--;
	}
 	mozileTarget.undoCounter--;
	var undoStep=undoStack[mozileTarget.undoCounter];

	var replacement=undoStep['node'];
	var undoTarget=mozileTarget.document.documentElement.childNodes[2];
	var parent=undoTarget.parentNode;
	parent.replaceChild(replacement,undoTarget);
	mozileUpdateUndoButtons();
} // end mozileUndo()



// Mozile Redo
// Moves up the undoStack from the current undoCounter,
// replacing the current BODY noe with the one in the stack.

function mozileRedo() {
	if(!mozileTarget.undoStack) {
		mozileTarget.undoStack=new Array();
		mozileTarget.undoCounter=0;
		return;
	}
	var undoStack=mozileTarget.undoStack;				// it is safe to copy this, since it will not be manipulated

	if(mozileTarget.undoCounter>undoStack.length-2) {
		//alert("Nothing to Redo!");
		return;
	}
 	mozileTarget.undoCounter++;
	var undoStep=undoStack[mozileTarget.undoCounter];

	var replacement=undoStep['node'];
	var undoTarget=mozileTarget.document.documentElement.childNodes[2];
	var parent=undoTarget.parentNode;
	parent.replaceChild(replacement,undoTarget);
	mozileUpdateUndoButtons();
} // end mozileRedo()



